@@ -13,16 +13,12 @@ require 5.004;
 
 package Gedcom::Item;
 
-use Data::Dumper;
-use Date::Manip;
-
-$Data::Dumper::Indent = 1;
-Date_Init("DateFormat=UK");
-
 BEGIN
 {
   use vars qw($VERSION);
-  $VERSION = "1.00";
+  $VERSION = "1.01";
+
+  eval "use Date::Manip";
 }
 
 sub new
@@ -30,8 +26,8 @@ sub new
   my $proto = shift;
   my $class = ref($proto) || $proto;
   my $self = { children => [], @_ };
-  bless($self, $class);
-  $self->read() if $self->{file};
+  bless $self, $class;
+  $self->read if $self->{file};
   $self;
 }
 
@@ -92,8 +88,6 @@ sub read
       push @{$self->{children}}, $structure;
       $count++;
     }
-    delete $structure->{gedcom};
-#   delete $structure->{grammar};
     return undef
       if $callback &&
          !&$callback($title, $txt1, "Record $count line " . $structure->{line},
@@ -133,7 +127,6 @@ sub add_children
     }
     else
     {
-      delete $next->{gedcom};
       push @{$record->{children}}, $next;
     }
   }
@@ -154,10 +147,10 @@ sub next_record
 #   print "line is $line";
     if (my ($structure) = $line =~ /^\s*(\w+): =\s*$/)
     {
-      $rec = $self->new(level      => -1,
-                        structure  => $structure,
-                        number     => "many",
-                        line       => $self->{fh}->input_line_number);
+      $rec = $self->new(level     => -1,
+                        structure => $structure,
+                        number    => "many",
+                        line      => $self->{fh}->input_line_number);
 #     print "found structure $structure\n";
     }
     elsif (my ($level, $xref, $tag, $value, $min, $max) =
@@ -200,12 +193,20 @@ sub next_record
                 \s*$/x)                    # optional whitespace at end
     {
       $rec = $self->new(line => $self->{fh}->input_line_number) unless $rec;
-      $rec->{level} = ($level eq "n" ? 0 : $level) if defined $level;
-      $rec->{xref}  = $xref                        if defined $xref;
-      $rec->{tag}   = $tag                         if defined $tag;
-      $rec->{value} = $value                       if defined $value;
-      $rec->{min}   = $min                         if defined $min;
-      $rec->{max}   = $max                         if defined $max;
+      $rec->{level}  = ($level eq "n" ? 0 : $level) if defined $level;
+      $rec->{xref}   = $xref                        if defined $xref;
+      $rec->{tag}    = $tag                         if defined $tag;
+      $rec->{value}  = $value                       if defined $value;
+      $rec->{min}    = $min                         if defined $min;
+      $rec->{max}    = $max                         if defined $max;
+      $rec->{gedcom} = $self->{gedcom};
+      if (ref($self) !~ /Grammar/ and $_ = $rec->{tag})
+      {
+        my $class = /INDI/i ? "Individual" :
+                    /FAM/i  ? "Family"     :
+                              undef;
+        bless $rec, "Gedcom::$class" if $class;
+      }
     }
     elsif ($line =~ /^\s*[\[\|\]]\s*(?:\/\*.*\*\/\s*)?$/)
     {
@@ -268,6 +269,11 @@ sub write
 sub normalise_dates
 {
   my $self = shift;
+  unless ($INC{"Date/Manip.pm"})
+  {
+    warn "Date::Manip is required to use normalise_dates()";
+    return;
+  }
   my $format = shift || "%A, %E %B %Y";
   if (defined $self->{tag} && $self->{tag} =~ /^date$/i)
   {
@@ -291,16 +297,6 @@ sub normalise_dates
   }
 }
 
-sub renumber
-{
-  my $self = shift;
-  return if exists $self->{new_xref};
-  for my $child (@{$self->{children}})
-  {
-    $child->renumber();
-  }
-}
-
 sub print
 {
   my $self = shift;
@@ -315,12 +311,12 @@ sub get_child
 {
   my $self = shift;
   my ($t) = @_;
-  my ($type, $count) = $t =~ /^_?(\w+?)(\d*)$/;
+  my ($tag, $count) = $t =~ /^_?(\w+?)(\d*)$/;
   $count = 1 unless $count;
-  # print "looking for <$type> number <$count>\n";
+  # print "looking for <$tag> number <$count>\n";
   for my $c (@{$self->{children}})
   {
-    return $c if $c->{tag} eq $type && !--$count;
+    return $c if $c->{tag} eq $tag && !--$count;
   }
   undef;
 }
@@ -328,8 +324,25 @@ sub get_child
 sub get_children
 {
   my $self = shift;
-  my ($type) = @_;
-  [ grep { $_->{tag} eq $type } @{$self->{children}} ]
+  my ($tag) = @_;
+  grep { $_->{tag} eq $tag } @{$self->{children}}
+}
+
+sub delete_child
+{
+  my $self = shift;
+  my ($child) = @_;
+  my $c = "$child";
+  my $n = 0;
+  for (@{$self->{children}})
+  {
+    my $ch = "$_";
+    # print "matching $ch against $c\n";
+    last if $c eq $ch;
+    $n++;
+  }
+  # print "deleting child $n of $#{$self->{children}}\n";
+  splice @{$self->{children}}, $n, 1;
 }
 
 1;
@@ -340,14 +353,157 @@ __END__
 
 Gedcom::Item - a base class for Gedcom::Grammar and Gedcom::Record
 
-Version 1.00 - 8th March 1999
+Version 1.01 - 27th April 1999
 
 =head1 SYNOPSIS
 
-use Gedcom::Record;
+  use Gedcom::Record;
+
+  $self->{grammar} = Gedcom::Grammar->new(file     => $self->{grammar_file},
+                                          callback => $self->{callback});
+  $self->read() if $self->{file};
+  $self->add_children($rec)
+  while (my $next = $self->next_record($record))
+  $line = $self->next_line()
+  my $line = $self->next_text_line()
+  $record->>write($fh, $level)
+  $record->normalise_dates($format)
+  $item->print()
+  my $child = get_child("CHIL2")
+  my @children = get_children("CHIL")
 
 =head1 DESCRIPTION
 
-To be written...
+A selection of subroutines to handle items in a gedcom file.
+
+=head1 HASH MEMBERS
+
+Some of the more important hash members are:
+
+=head2 $item->{level}
+
+The level of the item.
+
+=head2 $item->{xref}
+
+The cross reference, either hard or soft.
+
+=head2 $item->{tag}
+
+The name of the tag.
+
+=head2 $item->{value}
+
+The value of the item.
+
+=head2 $item->{min}
+
+The minimum number of items allowed.
+
+=head2 $item->{max}
+
+The maximum number of items allowed.
+
+=head2 $item->{children}
+
+Array of all children of this item.
+
+=head1 METHODS
+
+=head2 new
+
+  $self->{grammar} = Gedcom::Grammar->new(file     => $self->{grammar_file},
+                                          callback => $self->{callback});
+
+Create a new object.
+
+If file is supplied, it is the name of a file to read.
+
+If callback is supplied, it is a subroutine reference which is called at
+various times while the file is being read.
+
+The subroutine takes five parameters:
+  $title:     A title
+  $txt1:      One text message
+  $txt2:      A secondary text message
+  $current:   A count of how far through the file we are
+  $total:     The extent of the file
+
+The subroutine should return true iff the file shuld continue to be
+read.
+
+=head2 read
+
+  $self->read() if $self->{file};
+
+Read a file into the object.  Called by the constructor.
+
+=head2 add_children
+
+  $self->add_children($rec)
+
+Read in the children of a record.
+
+=head2 next_record
+
+  while (my $next = $self->next_record($record))
+
+Read the next record from a file.  Return the record or false if it
+cannot be read.
+
+=head2 next_line
+
+  $line = $self->next_line()
+
+Read the next line from the file, and return it or false.
+
+=head2 next_text_line
+
+  my $line = $self->next_text_line()
+
+Read the next line of text from the file, and return it or false.
+
+=head2 write
+
+  $record->>write($fh, $level)
+
+Write the record to a FileHandle.
+
+The subroutine takes two parameters:
+  $fh:        The FileHandle to which to write
+  $level:     The level of the record
+
+=head2 normalise_dates
+
+  $record->normalise_dates($format)
+
+Change the format of all dates in the record.
+
+See the documentation for Gedcom::normalise_dates
+
+=head2 print
+
+  $item->print()
+
+Print the item.  Used for debugging.  (What?  There are bugs?)
+
+=head2 get_child
+
+  my $child = get_child("CHIL2")
+
+Get a specific child from the item.
+
+The argument contains the name of the tag, and optionally the count.
+The regular expression to generate the tag and the count is:
+
+  my ($tag, $count) = $t =~ /^_?(\w+?)(\d*)$/;
+
+Returns the child, or undef if it doesn't exist;
+
+=head2 get_children
+
+  my @children = get_children("CHIL")
+
+Get all children matching a specified tag.
 
 =cut
